@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016-2023 gnome-mpv
+ * Copyright (c) 2016-2024 gnome-mpv
  *
  * This file is part of Celluloid.
  *
@@ -171,19 +171,22 @@ key_pressed_handler(	GtkEventControllerKey *key_controller,
 			gpointer data )
 
 {
-	CelluloidController *controller = data;
-	gchar *keystr = get_full_keystr(keyval, state);
-	gboolean searching = FALSE;
+	CelluloidController *controller =
+		data;
+	gchar *keystr =
+		get_full_keystr(keyval, state);
+	const gboolean playlist_visible =
+		celluloid_view_get_playlist_visible(controller->view);
+	const gboolean in_mpv_keymap =
+		celluloid_model_input_binding_exists(controller->model, keystr);
 
-	g_object_get(controller->view, "searching", &searching, NULL);
-
-	if(keystr && !searching)
+	if(keystr && !playlist_visible)
 	{
 		celluloid_model_key_down(controller->model, keystr);
 		g_free(keystr);
 	}
 
-	return TRUE;
+	return keystr && !playlist_visible && in_mpv_keymap;
 }
 
 static void
@@ -193,13 +196,14 @@ key_released_handler(	GtkEventControllerKey *key_controller,
 			GdkModifierType state,
 			gpointer data )
 {
-	CelluloidController *controller = data;
-	gchar *keystr = get_full_keystr(keyval, state);
-	gboolean searching = FALSE;
+	CelluloidController *controller =
+		data;
+	gchar *keystr =
+		get_full_keystr(keyval, state);
+	const gboolean playlist_visible =
+		celluloid_view_get_playlist_visible(controller->view);
 
-	g_object_get(controller->view, "searching", &searching, NULL);
-
-	if(keystr && !searching)
+	if(keystr && !playlist_visible)
 	{
 		celluloid_model_key_up(controller->model, keystr);
 		g_free(keystr);
@@ -345,8 +349,8 @@ celluloid_controller_input_connect_signals(CelluloidController *controller)
 				G_CALLBACK(key_released_handler),
 				controller );
 
-	GtkGesture *click_gesture =
-		gtk_gesture_click_new();
+	GtkGesture *const click_gesture =
+		controller->click_gesture;
 	gtk_gesture_single_set_button
 		(GTK_GESTURE_SINGLE(click_gesture), 0);
 	gtk_widget_add_controller
@@ -365,8 +369,8 @@ celluloid_controller_input_connect_signals(CelluloidController *controller)
 				G_CALLBACK(button_stopped_handler),
 				controller );
 
-	GtkEventController *motion_controller =
-		gtk_event_controller_motion_new();
+	GtkEventController *const motion_controller =
+		controller->motion_controller;
 	gtk_widget_add_controller
 		(GTK_WIDGET(video_area), GTK_EVENT_CONTROLLER(motion_controller));
 
@@ -375,9 +379,8 @@ celluloid_controller_input_connect_signals(CelluloidController *controller)
 				G_CALLBACK(mouse_move_handler),
 				controller );
 
-	GtkEventController *scroll_controller =
-		gtk_event_controller_scroll_new
-		(GTK_EVENT_CONTROLLER_SCROLL_BOTH_AXES | GTK_EVENT_CONTROLLER_SCROLL_DISCRETE);
+	GtkEventController *const scroll_controller =
+		controller->scroll_controller;
 	gtk_widget_add_controller
 		(GTK_WIDGET(video_area), GTK_EVENT_CONTROLLER(scroll_controller));
 
@@ -386,3 +389,19 @@ celluloid_controller_input_connect_signals(CelluloidController *controller)
 				G_CALLBACK(scroll_handler),
 				controller );
 }
+
+void
+celluloid_controller_input_disconnect_signals(CelluloidController *controller)
+{
+	CelluloidMainWindow *wnd =
+		celluloid_view_get_main_window(controller->view);
+	GtkWidget *video_area =
+		GTK_WIDGET(celluloid_main_window_get_video_area(wnd));
+
+	gtk_widget_remove_controller
+		(video_area, controller->motion_controller);
+	gtk_widget_remove_controller
+		(video_area, controller->scroll_controller);
+	gtk_widget_remove_controller
+		(video_area, GTK_EVENT_CONTROLLER(controller->click_gesture));
+}
