@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2023 gnome-mpv
+ * Copyright (c) 2017-2025 gnome-mpv
  *
  * This file is part of Celluloid.
  *
@@ -50,6 +50,7 @@ enum
 	PROP_TRACK_LIST,
 	PROP_DISC_LIST,
 	PROP_SKIP_ENABLED,
+	PROP_LOOP_FILE,
 	PROP_LOOP,
 	PROP_SHUFFLE,
 	PROP_MEDIA_TITLE,
@@ -77,6 +78,7 @@ struct _CelluloidView
 	GPtrArray *disc_list;
 	gboolean skip_enabled;
 	gboolean control_box_enabled;
+	gboolean loop_file;
 	gboolean loop;
 	gboolean shuffle;
 	gchar *media_title;
@@ -122,21 +124,12 @@ static void
 update_title(CelluloidView *view);
 
 static void
-show_message_dialog(	CelluloidView *view,
-			GtkMessageType type,
-			const gchar *title,
-			const gchar *prefix,
-			const gchar *msg );
+show_message_dialog(CelluloidView *view, const gchar *prefix, const gchar *msg);
 
 static void
 show_open_track_dialog(CelluloidView  *view, TrackType type);
 
 /* Dialog responses */
-static void
-message_dialog_response_handler(	GtkDialog *dialog,
-					gint response_id,
-					gpointer data);
-
 static void
 open_dialog_response_handler(GtkDialog *dialog, gint response_id, gpointer data);
 
@@ -158,6 +151,11 @@ save_playlist_response_handler(	GtkDialog *dialog,
 static gboolean
 mpv_reset_request_handler(AdwPreferencesWindow *dialog, gpointer data);
 
+static void
+error_handler(	CelluloidPreferencesDialog *dialog,
+		const gchar *message,
+		gpointer data );
+
 static void
 realize_handler(GtkWidget *widget, gpointer data);
 
@@ -232,12 +230,6 @@ constructed(GObject *object)
 	g_object_bind_property(	view, "skip-enabled",
 				control_box, "skip-enabled",
 				G_BINDING_DEFAULT );
-	g_object_bind_property(	view, "loop",
-				control_box, "loop",
-				G_BINDING_BIDIRECTIONAL );
-	g_object_bind_property(	view, "shuffle",
-				control_box, "shuffle",
-				G_BINDING_BIDIRECTIONAL );
 	g_object_bind_property(	view, "volume",
 				control_box, "volume",
 				G_BINDING_BIDIRECTIONAL );
@@ -250,6 +242,15 @@ constructed(GObject *object)
 	g_object_bind_property(	playlist, "searching",
 				view, "searching",
 				G_BINDING_BIDIRECTIONAL );
+	g_object_bind_property(	view, "loop-file",
+				playlist, "loop-file",
+				G_BINDING_BIDIRECTIONAL );
+	g_object_bind_property(	view, "loop",
+				playlist, "loop-playlist",
+				G_BINDING_BIDIRECTIONAL );
+	g_object_bind_property(	view, "shuffle",
+				playlist, "shuffle",
+				G_BINDING_BIDIRECTIONAL );
 
 	celluloid_main_window_load_state(wnd);
 	load_settings(view);
@@ -407,6 +408,10 @@ set_property(	GObject *object,
 		self->skip_enabled = g_value_get_boolean(value);
 		break;
 
+		case PROP_LOOP_FILE:
+		self->loop_file = g_value_get_boolean(value);
+		break;
+
 		case PROP_LOOP:
 		self->loop = g_value_get_boolean(value);
 		break;
@@ -500,6 +505,10 @@ get_property(	GObject *object,
 		g_value_set_boolean(value, self->skip_enabled);
 		break;
 
+		case PROP_LOOP_FILE:
+		g_value_set_boolean(value, self->loop_file);
+		break;
+
 		case PROP_LOOP:
 		g_value_set_boolean(value, self->loop);
 		break;
@@ -530,14 +539,42 @@ static void
 load_css(CelluloidView *view)
 {
 	const gchar *style =
-		"celluloid-seek-bar slider { border-radius: 100%; }"
-		"celluloid-video-area { background-color: black; }"
-		".dialog-action-area { margin: -12px 12px 12px 0px; }"
-		".dialog-action-area > button { margin-left: 12px; }";
-	GtkCssProvider *style_provider =
-		gtk_css_provider_new();
+		"celluloid-video-area"
+		"{"
+		"	background-color: var(--view-bg-color);"
+		"}"
+		"celluloid-seek-bar slider"
+		"{"
+		"	border-radius: 100%;"
+		"}"
+		".dialog-action-area"
+		"{"
+		"	margin: -12px 12px 12px 0px;"
+		"}"
+		".dialog-action-area > button"
+		"{"
+		"	margin-left: 12px;"
+		"}"
+		".osd.docked"
+		"{"
+		"	border-radius: 0px;"
+		"}"
+		".control-box.osd.undocked"
+		"{"
+		"	margin: 12px 12px 12px 12px;"
+		"}"
+		".control-box.osd.docked"
+		"{"
+		"	margin: 0px 0px 0px 0px;"
+		"}"
+		".osd, .floating-header windowcontrols image"
+		"{"
+		"	border: 1px solid var(--border-color);"
+		"}";
 
-	gtk_css_provider_load_from_data(style_provider, style, -1);
+	GtkCssProvider *style_provider = gtk_css_provider_new();
+
+	gtk_css_provider_load_from_string(style_provider, style);
 
 	GdkSurface *surface = gtk_widget_get_surface(view);
 
@@ -568,12 +605,7 @@ load_settings(CelluloidView *view)
 	CelluloidControlBox *control_box =
 		celluloid_main_window_get_control_box(wnd);
 
-	gboolean csd_enable;
-
-	csd_enable =	g_settings_get_boolean (settings, "csd-enable");
-
 	g_object_set(	control_box,
-			"show-fullscreen-button", !csd_enable,
 			"skip-enabled", FALSE,
 			NULL );
 
@@ -639,18 +671,23 @@ update_title(CelluloidView *view)
 	gchar *title =
 		sanitize_utf8(title_source, TRUE);
 
+	CelluloidMainWindow *wnd =
+		CELLULOID_MAIN_WINDOW(view);
+	CelluloidVideoArea *video_area =
+		celluloid_main_window_get_video_area(wnd);
+	CelluloidControlBox *control_box =
+		celluloid_video_area_get_control_box
+		(CELLULOID_VIDEO_AREA(video_area));
+
+	celluloid_control_box_set_title(control_box, title);
 	gtk_window_set_title(GTK_WINDOW(view), title);
 	g_free(title);
 }
 
 void
-show_message_dialog(	CelluloidView *view,
-			GtkMessageType type,
-			const gchar *title,
-			const gchar *prefix,
-			const gchar *msg )
+show_message_dialog(CelluloidView *view, const gchar *prefix, const gchar *msg)
 {
-	GtkWidget *dialog = NULL;
+	GtkAlertDialog *dialog = NULL;
 
 	if(view->has_dialog)
 	{
@@ -658,11 +695,11 @@ show_message_dialog(	CelluloidView *view,
 
 		if(prefix)
 		{
-			g_info("Content: *%s* [%s] %s", title, prefix, msg);
+			g_info("Content: [%s] %s", prefix, msg);
 		}
 		else
 		{
-			g_info("Content: *%s* %s", title, msg);
+			g_info("Content: %s", msg);
 		}
 	}
 	else if(prefix)
@@ -670,61 +707,22 @@ show_message_dialog(	CelluloidView *view,
 		gchar *prefix_escaped = g_markup_printf_escaped("%s", prefix);
 		gchar *msg_escaped = g_markup_printf_escaped("%s", msg);
 
-		dialog =	gtk_message_dialog_new_with_markup
-				(	GTK_WINDOW(view),
-					GTK_DIALOG_DESTROY_WITH_PARENT,
-					type,
-					GTK_BUTTONS_OK,
-					"<b>[%s]</b> %s",
-					prefix_escaped,
-					msg_escaped );
+		dialog =	gtk_alert_dialog_new
+				("[%s] %s", prefix_escaped, msg_escaped);
 
 		g_free(prefix_escaped);
 		g_free(msg_escaped);
 	}
 	else
 	{
-		dialog =	gtk_message_dialog_new
-				(	GTK_WINDOW(view),
-					GTK_DIALOG_DESTROY_WITH_PARENT,
-					type,
-					GTK_BUTTONS_OK,
-					"%s",
-					msg );
+		dialog = gtk_alert_dialog_new("%s", msg);
 	}
 
 	if(dialog)
 	{
-		GtkMessageDialog *message_dialog =
-			GTK_MESSAGE_DIALOG(dialog);
-		GtkWidget *message_area =
-			gtk_message_dialog_get_message_area(message_dialog);
-		GtkWidget *first_child =
-			gtk_widget_get_first_child(message_area);
-
 		view->has_dialog = TRUE;
 
-		for(	GtkWidget *child = first_child;
-			child;
-			child = gtk_widget_get_next_sibling(child) )
-		{
-			if(GTK_IS_LABEL(child))
-			{
-				gtk_label_set_wrap_mode
-					(	GTK_LABEL(child),
-						PANGO_WRAP_WORD_CHAR );
-			}
-		}
-
-		g_signal_connect
-			(	dialog,
-				"response",
-				G_CALLBACK(message_dialog_response_handler),
-				view );
-
-		gtk_window_set_title(GTK_WINDOW(dialog), title);
-		gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
-		gtk_widget_set_visible(dialog, TRUE);
+		gtk_alert_dialog_show(dialog, GTK_WINDOW(view));
 	}
 }
 
@@ -777,16 +775,6 @@ show_open_track_dialog(CelluloidView  *view, TrackType type)
 	celluloid_file_chooser_show(chooser);
 }
 
-static void
-message_dialog_response_handler(	GtkDialog *dialog,
-					gint response_id,
-					gpointer data)
-{
-	CELLULOID_VIEW(data)->has_dialog = FALSE;
-
-	gtk_window_destroy(GTK_WINDOW(dialog));
-}
-
 static void
 open_dialog_response_handler(GtkDialog *dialog, gint response_id, gpointer data)
 {
@@ -796,8 +784,8 @@ open_dialog_response_handler(GtkDialog *dialog, gint response_id, gpointer data)
 
 	if(response_id == GTK_RESPONSE_ACCEPT)
 	{
-		GtkFileChooser *chooser = GTK_FILE_CHOOSER(dialog);
-		GListModel *files = gtk_file_chooser_get_files(chooser);
+		CelluloidFileChooser *chooser = CELLULOID_FILE_CHOOSER(dialog);
+		GListModel *files = celluloid_file_chooser_get_files(chooser);
 
 		if(files)
 		{
@@ -851,8 +839,8 @@ open_track_dialog_response_handler(	GtkDialog *dialog,
 {
 	if(response_id == GTK_RESPONSE_ACCEPT)
 	{
-		GtkFileChooser *chooser = GTK_FILE_CHOOSER(dialog);
-		GListModel *files = gtk_file_chooser_get_files(chooser);
+		CelluloidFileChooser *chooser = CELLULOID_FILE_CHOOSER(dialog);
+		GListModel *files = celluloid_file_chooser_get_files(chooser);
 		const gchar *name = NULL;
 
 		TrackType type =
@@ -908,7 +896,8 @@ save_playlist_response_handler(	GtkDialog *dialog,
 	if(response_id == GTK_RESPONSE_ACCEPT)
 	{
 		/* There should be only one file selected. */
-		dest_file = gtk_file_chooser_get_file(GTK_FILE_CHOOSER(dialog));
+		dest_file =	celluloid_file_chooser_get_file
+				(CELLULOID_FILE_CHOOSER(dialog));
 	}
 
 	celluloid_file_chooser_destroy(CELLULOID_FILE_CHOOSER(dialog));
@@ -921,11 +910,7 @@ save_playlist_response_handler(	GtkDialog *dialog,
 
 	if(error)
 	{
-		show_message_dialog(	view,
-					GTK_MESSAGE_ERROR,
-					_("Error"),
-					NULL,
-					error->message );
+		show_message_dialog(view, NULL, error->message);
 
 		g_error_free(error);
 	}
@@ -945,8 +930,6 @@ mpv_reset_request_handler(AdwPreferencesWindow *dialog, gpointer data)
 	if(celluloid_main_window_get_csd_enabled(wnd) != csd_enable)
 	{
 		show_message_dialog(	CELLULOID_VIEW(data),
-					GTK_MESSAGE_INFO,
-					g_get_application_name(),
 					NULL,
 					_("Enabling or disabling "
 					"client-side decorations "
@@ -962,6 +945,16 @@ mpv_reset_request_handler(AdwPreferencesWindow *dialog, gpointer data)
 	return FALSE;
 }
 
+static void
+error_handler(	CelluloidPreferencesDialog *dialog,
+		const gchar *message,
+		gpointer data )
+{
+	CelluloidView *view = CELLULOID_VIEW(data);
+
+	show_message_dialog(view, NULL, message);
+}
+
 static void
 realize_handler(GtkWidget *widget, gpointer data)
 {
@@ -1225,6 +1218,14 @@ celluloid_view_class_init(CelluloidViewClass *klass)
 			G_PARAM_READWRITE );
 	g_object_class_install_property(object_class, PROP_SKIP_ENABLED, pspec);
 
+	pspec = g_param_spec_boolean
+		(	"loop-file",
+			"Loop file",
+			"Whether or not the the loop file button is active",
+			FALSE,
+			G_PARAM_READWRITE );
+	g_object_class_install_property(object_class, PROP_LOOP_FILE, pspec);
+
 	pspec = g_param_spec_boolean
 		(	"loop",
 			"Loop",
@@ -1407,6 +1408,7 @@ celluloid_view_init(CelluloidView *view)
 	view->track_list = NULL;
 	view->disc_list = NULL;
 	view->skip_enabled = FALSE;
+	view->loop_file = FALSE;
 	view->loop = FALSE;
 	view->shuffle = FALSE;
 	view->media_title = NULL;
@@ -1549,16 +1551,14 @@ void
 celluloid_view_show_save_playlist_dialog(CelluloidView *view)
 {
 	CelluloidFileChooser *file_chooser;
-	GtkFileChooser *gtk_chooser;
 
 	file_chooser =	celluloid_file_chooser_new
 			(	_("Save Playlist"),
 				GTK_WINDOW(view),
 				GTK_FILE_CHOOSER_ACTION_SAVE,
 				TRUE );
-	gtk_chooser = GTK_FILE_CHOOSER(file_chooser);
 
-	gtk_file_chooser_set_current_name(gtk_chooser, "playlist.m3u");
+	celluloid_file_chooser_set_current_name(file_chooser, "playlist.m3u");
 
 	g_signal_connect(	file_chooser,
 				"response",
@@ -1577,8 +1577,13 @@ celluloid_view_show_preferences_dialog(CelluloidView *view)
 				"mpv-reset-request",
 				G_CALLBACK(mpv_reset_request_handler),
 				view );
+	g_signal_connect(	dialog,
+				"error-raised",
+				G_CALLBACK(error_handler),
+				view );
 
-	gtk_widget_set_visible(dialog, TRUE);
+	celluloid_preferences_dialog_present
+		(CELLULOID_PREFERENCES_DIALOG(dialog));
 }
 
 void
@@ -1594,7 +1599,7 @@ celluloid_view_show_about_window (CelluloidView *view)
 {
 	const gchar *const authors[] = AUTHORS;
 
-	adw_show_about_window(	GTK_WINDOW(view),
+	adw_show_about_dialog(	GTK_WIDGET(view),
 				"application-icon",
 				ICON_NAME,
 				"application-name",
@@ -1685,12 +1690,9 @@ celluloid_view_get_video_area_geometry(	CelluloidView *view,
 {
 	CelluloidMainWindow *wnd = CELLULOID_MAIN_WINDOW(view);
 	CelluloidVideoArea *area = celluloid_main_window_get_video_area(wnd);
-	GtkAllocation allocation;
-
-	gtk_widget_get_allocation(GTK_WIDGET(area), &allocation);
-
-	*width = allocation.width;
-	*height = allocation.height;
+	GtkGLArea *gl_area = celluloid_video_area_get_gl_area(area);
+	*width = gtk_widget_get_width(GTK_WIDGET(gl_area));
+	*height = gtk_widget_get_height(GTK_WIDGET(gl_area));
 }
 
 void
@@ -1761,7 +1763,7 @@ celluloid_view_set_playlist_visible(CelluloidView *view, gboolean visible)
 gboolean
 celluloid_view_get_playlist_visible(CelluloidView *view)
 {
-	return 	celluloid_main_window_get_playlist_visible
+	return	celluloid_main_window_get_playlist_visible
 		(CELLULOID_MAIN_WINDOW(view));
 }
 
